@@ -16,132 +16,150 @@ import (
 	"github.com/rivo/uniseg"
 )
 
-const Usage = `usage: fzf [options]
+const Usage = `fzf is an interactive filter program for any kind of list.
+
+It implements a "fuzzy" matching algorithm, so you can quickly type in patterns
+with omitted characters and still get the results you want.
+
+Project URL: https://github.com/junegunn/fzf
+Author: Junegunn Choi <junegunn.c@gmail.com>
+
+Usage: fzf [options]
 
   Search
-    -x, --extended         Extended-search mode
-                           (enabled by default; +x or --no-extended to disable)
-    -e, --exact            Enable Exact-match
-    -i, --ignore-case      Case-insensitive match (default: smart-case match)
-    +i, --no-ignore-case   Case-sensitive match
-    --scheme=SCHEME        Scoring scheme [default|path|history]
-    --literal              Do not normalize latin script letters before matching
-    -n, --nth=N[,..]       Comma-separated list of field index expressions
-                           for limiting search scope. Each can be a non-zero
-                           integer or a range expression ([BEGIN]..[END]).
-    --with-nth=N[,..]      Transform the presentation of each line using
-                           field index expressions
-    -d, --delimiter=STR    Field delimiter regex (default: AWK-style)
-    +s, --no-sort          Do not sort the result
-    --track                Track the current selection when the result is updated
-    --tac                  Reverse the order of the input
-    --disabled             Do not perform search
-    --tiebreak=CRI[,..]    Comma-separated list of sort criteria to apply
-                           when the scores are tied [length|chunk|begin|end|index]
-                           (default: length)
-
+    -x, --extended          Extended-search mode
+                            (enabled by default; +x or --no-extended to disable)
+    -e, --exact             Enable Exact-match
+    -i, --ignore-case       Case-insensitive match (default: smart-case match)
+    +i, --no-ignore-case    Case-sensitive match
+    --scheme=SCHEME         Scoring scheme [default|path|history]
+    --literal               Do not normalize latin script letters before matching
+    -n, --nth=N[,..]        Comma-separated list of field index expressions
+                            for limiting search scope. Each can be a non-zero
+                            integer or a range expression ([BEGIN]..[END]).
+    --with-nth=N[,..]       Transform the presentation of each line using
+                            field index expressions
+    -d, --delimiter=STR     Field delimiter regex (default: AWK-style)
+    +s, --no-sort           Do not sort the result
+    --tail=NUM              Maximum number of items to keep in memory
+    --track                 Track the current selection when the result is updated
+    --tac                   Reverse the order of the input
+    --disabled              Do not perform search
+    --tiebreak=CRI[,..]     Comma-separated list of sort criteria to apply
+                            when the scores are tied [length|chunk|begin|end|index]
+                            (default: length)
   Interface
-    -m, --multi[=MAX]      Enable multi-select with tab/shift-tab
-    --no-mouse             Disable mouse
-    --bind=KEYBINDS        Custom key bindings. Refer to the man page.
-    --cycle                Enable cyclic scroll
-    --keep-right           Keep the right end of the line visible on overflow
-    --scroll-off=LINES     Number of screen lines to keep above or below when
-                           scrolling to the top or to the bottom (default: 0)
-    --no-hscroll           Disable horizontal scroll
-    --hscroll-off=COLS     Number of screen columns to keep to the right of the
-                           highlighted substring (default: 10)
-    --filepath-word        Make word-wise movements respect path separators
-    --jump-labels=CHARS    Label characters for jump mode
+    -m, --multi[=MAX]       Enable multi-select with tab/shift-tab
+    --no-mouse              Disable mouse
+    --bind=KEYBINDS         Custom key bindings. Refer to the man page.
+    --cycle                 Enable cyclic scroll
+    --no-multi-line         Disable multi-line display of items when using --read0
+    --keep-right            Keep the right end of the line visible on overflow
+    --scroll-off=LINES      Number of screen lines to keep above or below when
+                            scrolling to the top or to the bottom (default: 0)
+    --no-hscroll            Disable horizontal scroll
+    --hscroll-off=COLS      Number of screen columns to keep to the right of the
+                            highlighted substring (default: 10)
+    --filepath-word         Make word-wise movements respect path separators
+    --jump-labels=CHARS     Label characters for jump mode
 
   Layout
-    --height=[~]HEIGHT[%]  Display fzf window below the cursor with the given
-                           height instead of using fullscreen.
-                           A negative value is calculated as the terminal height
-                           minus the given value.
-                           If prefixed with '~', fzf will determine the height
-                           according to the input size.
-    --min-height=HEIGHT    Minimum height when --height is given in percent
-                           (default: 10)
-    --layout=LAYOUT        Choose layout: [default|reverse|reverse-list]
-    --border[=STYLE]       Draw border around the finder
-                           [rounded|sharp|bold|block|thinblock|double|horizontal|vertical|
-                            top|bottom|left|right|none] (default: rounded)
-    --border-label=LABEL   Label to print on the border
-    --border-label-pos=COL Position of the border label
-                           [POSITIVE_INTEGER: columns from left|
-                            NEGATIVE_INTEGER: columns from right][:bottom]
-                           (default: 0 or center)
-    --margin=MARGIN        Screen margin (TRBL | TB,RL | T,RL,B | T,R,B,L)
-    --padding=PADDING      Padding inside border (TRBL | TB,RL | T,RL,B | T,R,B,L)
-    --info=STYLE           Finder info style
-                           [default|right|hidden|inline[-right][:PREFIX]]
-    --separator=STR        String to form horizontal separator on info line
-    --no-separator         Hide info line separator
-    --scrollbar[=C1[C2]]   Scrollbar character(s) (each for main and preview window)
-    --no-scrollbar         Hide scrollbar
-    --prompt=STR           Input prompt (default: '> ')
-    --pointer=STR          Pointer to the current line (default: '>')
-    --marker=STR           Multi-select marker (default: '>')
-    --header=STR           String to print as header
-    --header-lines=N       The first N lines of the input are treated as header
-    --header-first         Print header before the prompt line
-    --ellipsis=STR         Ellipsis to show when line is truncated (default: '..')
+    --height=[~]HEIGHT[%]   Display fzf window below the cursor with the given
+                            height instead of using fullscreen.
+                            A negative value is calculated as the terminal height
+                            minus the given value.
+                            If prefixed with '~', fzf will determine the height
+                            according to the input size.
+    --min-height=HEIGHT     Minimum height when --height is given in percent
+                            (default: 10)
+    --tmux[=OPTS]           Start fzf in a tmux popup (requires tmux 3.3+)
+                            [center|top|bottom|left|right][,SIZE[%]][,SIZE[%]]
+                            (default: center,50%)
+    --layout=LAYOUT         Choose layout: [default|reverse|reverse-list]
+    --border[=STYLE]        Draw border around the finder
+                            [rounded|sharp|bold|block|thinblock|double|horizontal|vertical|
+                             top|bottom|left|right|none] (default: rounded)
+    --border-label=LABEL    Label to print on the border
+    --border-label-pos=COL  Position of the border label
+                            [POSITIVE_INTEGER: columns from left|
+                             NEGATIVE_INTEGER: columns from right][:bottom]
+                            (default: 0 or center)
+    --margin=MARGIN         Screen margin (TRBL | TB,RL | T,RL,B | T,R,B,L)
+    --padding=PADDING       Padding inside border (TRBL | TB,RL | T,RL,B | T,R,B,L)
+    --info=STYLE            Finder info style
+                            [default|right|hidden|inline[-right][:PREFIX]]
+    --separator=STR         String to form horizontal separator on info line
+    --no-separator          Hide info line separator
+    --scrollbar[=C1[C2]]    Scrollbar character(s) (each for main and preview window)
+    --no-scrollbar          Hide scrollbar
+    --prompt=STR            Input prompt (default: '> ')
+    --pointer=STR           Pointer to the current line (default: '▌' or '>')
+    --marker=STR            Multi-select marker (default: '┃' or '>')
+    --marker-multi-line=STR Multi-select marker for multi-line entries;
+                            3 elements for top, middle, and bottom (default: '╻┃╹')
+    --header=STR            String to print as header
+    --header-lines=N        The first N lines of the input are treated as header
+    --header-first          Print header before the prompt line
+    --ellipsis=STR          Ellipsis to show when line is truncated (default: '..')
 
   Display
-    --ansi                 Enable processing of ANSI color codes
-    --tabstop=SPACES       Number of spaces for a tab character (default: 8)
-    --color=COLSPEC        Base scheme (dark|light|16|bw) and/or custom colors
-    --highlight-line       Highlight the whole current line
-    --no-bold              Do not use bold text
+    --ansi                  Enable processing of ANSI color codes
+    --tabstop=SPACES        Number of spaces for a tab character (default: 8)
+    --color=COLSPEC         Base scheme (dark|light|16|bw) and/or custom colors
+    --highlight-line        Highlight the whole current line
+    --no-bold               Do not use bold text
 
   History
-    --history=FILE         History file
-    --history-size=N       Maximum number of history entries (default: 1000)
+    --history=FILE          History file
+    --history-size=N        Maximum number of history entries (default: 1000)
 
   Preview
-    --preview=COMMAND      Command to preview highlighted line ({})
-    --preview-window=OPT   Preview window layout (default: right:50%)
-                           [up|down|left|right][,SIZE[%]]
-                           [,[no]wrap][,[no]cycle][,[no]follow][,[no]hidden]
-                           [,border-BORDER_OPT]
-                           [,+SCROLL[OFFSETS][/DENOM]][,~HEADER_LINES]
-                           [,default][,<SIZE_THRESHOLD(ALTERNATIVE_LAYOUT)]
+    --preview=COMMAND       Command to preview highlighted line ({})
+    --preview-window=OPT    Preview window layout (default: right:50%)
+                            [up|down|left|right][,SIZE[%]]
+                            [,[no]wrap][,[no]cycle][,[no]follow][,[no]hidden]
+                            [,border-BORDER_OPT]
+                            [,+SCROLL[OFFSETS][/DENOM]][,~HEADER_LINES]
+                            [,default][,<SIZE_THRESHOLD(ALTERNATIVE_LAYOUT)]
     --preview-label=LABEL
-    --preview-label-pos=N  Same as --border-label and --border-label-pos,
-                           but for preview window
+    --preview-label-pos=N   Same as --border-label and --border-label-pos,
+                            but for preview window
 
   Scripting
-    -q, --query=STR        Start the finder with the given query
-    -1, --select-1         Automatically select the only match
-    -0, --exit-0           Exit immediately when there's no match
-    -f, --filter=STR       Filter mode. Do not start interactive finder.
-    --print-query          Print query as the first line
-    --expect=KEYS          Comma-separated list of keys to complete fzf
-    --read0                Read input delimited by ASCII NUL characters
-    --print0               Print output delimited by ASCII NUL characters
-    --sync                 Synchronous search for multi-staged filtering
-    --with-shell=STR       Shell command and flags to start child processes with
-    --listen[=[ADDR:]PORT] Start HTTP server to receive actions (POST /)
-                           (To allow remote process execution, use --listen-unsafe)
-    --version              Display version information and exit
+    -q, --query=STR         Start the finder with the given query
+    -1, --select-1          Automatically select the only match
+    -0, --exit-0            Exit immediately when there's no match
+    -f, --filter=STR        Filter mode. Do not start interactive finder.
+    --print-query           Print query as the first line
+    --expect=KEYS           Comma-separated list of keys to complete fzf
+    --read0                 Read input delimited by ASCII NUL characters
+    --print0                Print output delimited by ASCII NUL characters
+    --sync                  Synchronous search for multi-staged filtering
+    --with-shell=STR        Shell command and flags to start child processes with
+    --listen[=[ADDR:]PORT]  Start HTTP server to receive actions (POST /)
+                            (To allow remote process execution, use --listen-unsafe)
 
-  Directory traversal      (Only used when $FZF_DEFAULT_COMMAND is not set)
-    --walker=OPTS          [file][,dir][,follow][,hidden] (default: file,follow,hidden)
-    --walker-root=DIR      Root directory from which to start walker (default: .)
-    --walker-skip=DIRS     Comma-separated list of directory names to skip
-                           (default: .git,node_modules)
+  Directory traversal       (Only used when $FZF_DEFAULT_COMMAND is not set)
+    --walker=OPTS           [file][,dir][,follow][,hidden] (default: file,follow,hidden)
+    --walker-root=DIR       Root directory from which to start walker (default: .)
+    --walker-skip=DIRS      Comma-separated list of directory names to skip
+                            (default: .git,node_modules)
 
   Shell integration
-    --bash                 Print script to set up Bash shell integration
-    --zsh                  Print script to set up Zsh shell integration
-    --fish                 Print script to set up Fish shell integration
+    --bash                  Print script to set up Bash shell integration
+    --zsh                   Print script to set up Zsh shell integration
+    --fish                  Print script to set up Fish shell integration
+
+  Help
+    --version               Display version information and exit
+    --help                  Show this message
+    --man                   Show man page
 
   Environment variables
-    FZF_DEFAULT_COMMAND    Default command to use when input is tty
-    FZF_DEFAULT_OPTS       Default options (e.g. '--layout=reverse --info=inline')
-    FZF_DEFAULT_OPTS_FILE  Location of the file to read default options from
-    FZF_API_KEY            X-API-Key header for HTTP server (--listen)
+    FZF_DEFAULT_COMMAND     Default command to use when input is tty
+    FZF_DEFAULT_OPTS        Default options (e.g. '--layout=reverse --info=inline')
+    FZF_DEFAULT_OPTS_FILE   Location of the file to read default options from
+    FZF_API_KEY             X-API-Key header for HTTP server (--listen)
 
 `
 
@@ -173,6 +191,7 @@ type heightSpec struct {
 	percent bool
 	auto    bool
 	inverse bool
+	index   int
 }
 
 type sizeSpec struct {
@@ -180,6 +199,13 @@ type sizeSpec struct {
 	percent bool
 }
 
+func (s sizeSpec) String() string {
+	if s.percent {
+		return fmt.Sprintf("%d%%", int(s.size))
+	}
+	return fmt.Sprintf("%d", int(s.size))
+}
+
 func defaultMargin() [4]sizeSpec {
 	return [4]sizeSpec{}
 }
@@ -199,8 +225,16 @@ const (
 	posDown
 	posLeft
 	posRight
+	posCenter
 )
 
+type tmuxOptions struct {
+	width    sizeSpec
+	height   sizeSpec
+	position windowPosition
+	index    int
+}
+
 type layoutType int
 
 const (
@@ -248,6 +282,77 @@ func (o *previewOpts) Toggle() {
 	o.hidden = !o.hidden
 }
 
+func defaultTmuxOptions(index int) *tmuxOptions {
+	return &tmuxOptions{
+		position: posCenter,
+		width:    sizeSpec{50, true},
+		height:   sizeSpec{50, true},
+		index:    index}
+}
+
+func parseTmuxOptions(arg string, index int) (*tmuxOptions, error) {
+	var err error
+	opts := defaultTmuxOptions(index)
+	tokens := splitRegexp.Split(arg, -1)
+	errorToReturn := errors.New("invalid tmux option: " + arg + " (expected: [center|top|bottom|left|right][,SIZE[%]][,SIZE[%]])")
+	if len(tokens) == 0 || len(tokens) > 3 {
+		return nil, errorToReturn
+	}
+
+	// Defaults to 'center'
+	switch tokens[0] {
+	case "top", "up":
+		opts.position = posUp
+		opts.width = sizeSpec{100, true}
+	case "bottom", "down":
+		opts.position = posDown
+		opts.width = sizeSpec{100, true}
+	case "left":
+		opts.position = posLeft
+		opts.height = sizeSpec{100, true}
+	case "right":
+		opts.position = posRight
+		opts.height = sizeSpec{100, true}
+	case "center":
+	default:
+		tokens = append([]string{"center"}, tokens...)
+	}
+
+	// One size given
+	var size1 sizeSpec
+	if len(tokens) > 1 {
+		if size1, err = parseSize(tokens[1], 100, "size"); err != nil {
+			return nil, errorToReturn
+		}
+	}
+
+	// Two sizes given
+	var size2 sizeSpec
+	if len(tokens) == 3 {
+		if size2, err = parseSize(tokens[2], 100, "size"); err != nil {
+			return nil, errorToReturn
+		}
+		opts.width = size1
+		opts.height = size2
+	} else if len(tokens) == 2 {
+		switch tokens[0] {
+		case "top", "up":
+			opts.height = size1
+		case "bottom", "down":
+			opts.height = size1
+		case "left":
+			opts.width = size1
+		case "right":
+			opts.width = size1
+		case "center":
+			opts.width = size1
+			opts.height = size1
+		}
+	}
+
+	return opts, nil
+}
+
 func parseLabelPosition(opts *labelOpts, arg string) error {
 	opts.column = 0
 	opts.bottom = false
@@ -296,9 +401,14 @@ type walkerOpts struct {
 type Options struct {
 	Input        chan string
 	Output       chan string
+	NoWinpty     bool
+	Tmux         *tmuxOptions
+	ForceTtyIn   bool
+	ProxyScript  string
 	Bash         bool
 	Zsh          bool
 	Fish         bool
+	Man          bool
 	Fuzzy        bool
 	FuzzyAlgo    algo.Algo
 	Scheme       string
@@ -312,6 +422,7 @@ type Options struct {
 	Sort         int
 	Track        trackOption
 	Tac          bool
+	Tail         int
 	Criteria     []criterion
 	Multi        int
 	Ansi         bool
@@ -323,6 +434,7 @@ type Options struct {
 	MinHeight    int
 	Layout       layoutType
 	Cycle        bool
+	MultiLine    bool
 	CursorLine   bool
 	KeepRight    bool
 	Hscroll      bool
@@ -334,8 +446,9 @@ type Options struct {
 	Separator    *string
 	JumpLabels   string
 	Prompt       string
-	Pointer      string
-	Marker       string
+	Pointer      *string
+	Marker       *string
+	MarkerMulti  [3]string
 	Query        string
 	Select1      bool
 	Exit0        bool
@@ -393,10 +506,18 @@ func defaultPreviewOpts(command string) previewOpts {
 }
 
 func defaultOptions() *Options {
+	var theme *tui.ColorTheme
+	if os.Getenv("NO_COLOR") != "" {
+		theme = tui.NoColorTheme()
+	} else {
+		theme = tui.EmptyTheme()
+	}
+
 	return &Options{
 		Bash:         false,
 		Zsh:          false,
 		Fish:         false,
+		Man:          false,
 		Fuzzy:        true,
 		FuzzyAlgo:    algo.FuzzyMatchV2,
 		Scheme:       "default",
@@ -414,23 +535,24 @@ func defaultOptions() *Options {
 		Multi:        0,
 		Ansi:         false,
 		Mouse:        true,
-		Theme:        tui.EmptyTheme(),
+		Theme:        theme,
 		Black:        false,
 		Bold:         true,
 		MinHeight:    10,
 		Layout:       layoutDefault,
 		Cycle:        false,
+		MultiLine:    true,
 		KeepRight:    false,
 		Hscroll:      true,
 		HscrollOff:   10,
-		ScrollOff:    0,
+		ScrollOff:    3,
 		FileWord:     false,
 		InfoStyle:    infoDefault,
 		Separator:    nil,
 		JumpLabels:   defaultJumpLabels,
 		Prompt:       "> ",
-		Pointer:      ">",
-		Marker:       ">",
+		Pointer:      nil,
+		Marker:       nil,
 		Query:        "",
 		Select1:      false,
 		Exit0:        false,
@@ -554,7 +676,7 @@ func splitNth(str string) ([]Range, error) {
 
 func delimiterRegexp(str string) Delimiter {
 	// Special handling of \t
-	str = strings.Replace(str, "\\t", "\t", -1)
+	str = strings.ReplaceAll(str, "\\t", "\t")
 
 	// 1. Pattern does not contain any special character
 	if regexp.QuoteMeta(str) == str {
@@ -1077,7 +1199,7 @@ const (
 
 func init() {
 	executeRegexp = regexp.MustCompile(
-		`(?si)[:+](become|execute(?:-multi|-silent)?|reload(?:-sync)?|preview|(?:change|transform)-(?:header|query|prompt|border-label|preview-label)|transform|change-(?:preview-window|preview|multi)|(?:re|un)bind|pos|put)`)
+		`(?si)[:+](become|execute(?:-multi|-silent)?|reload(?:-sync)?|preview|(?:change|transform)-(?:header|query|prompt|border-label|preview-label)|transform|change-(?:preview-window|preview|multi)|(?:re|un)bind|pos|put|print)`)
 	splitRegexp = regexp.MustCompile("[,:]+")
 	actionNameRegexp = regexp.MustCompile("(?i)^[a-z-]+")
 }
@@ -1132,9 +1254,9 @@ Loop:
 		masked += strings.Repeat(" ", loc[1])
 		action = action[loc[1]:]
 	}
-	masked = strings.Replace(masked, "::", string([]rune{escapedColon, ':'}), -1)
-	masked = strings.Replace(masked, ",:", string([]rune{escapedComma, ':'}), -1)
-	masked = strings.Replace(masked, "+:", string([]rune{escapedPlus, ':'}), -1)
+	masked = strings.ReplaceAll(masked, "::", string([]rune{escapedColon, ':'}))
+	masked = strings.ReplaceAll(masked, ",:", string([]rune{escapedComma, ':'}))
+	masked = strings.ReplaceAll(masked, "+:", string([]rune{escapedPlus, ':'}))
 	return masked
 }
 
@@ -1449,6 +1571,8 @@ func isExecuteAction(str string) actionType {
 		return actExecuteSilent
 	case "execute-multi":
 		return actExecuteMulti
+	case "print":
+		return actPrint
 	case "put":
 		return actPut
 	case "transform":
@@ -1516,8 +1640,8 @@ func parseSize(str string, maxPercent float64, label string) (sizeSpec, error) {
 	return sizeSpec{val, percent}, nil
 }
 
-func parseHeight(str string) (heightSpec, error) {
-	heightSpec := heightSpec{}
+func parseHeight(str string, index int) (heightSpec, error) {
+	heightSpec := heightSpec{index: index}
 	if strings.HasPrefix(str, "~") {
 		heightSpec.auto = true
 		str = str[1:]
@@ -1734,7 +1858,38 @@ func parseMargin(opt string, margin string) ([4]sizeSpec, error) {
 	return [4]sizeSpec{}, errors.New("invalid " + opt + ": " + margin)
 }
 
-func parseOptions(opts *Options, allArgs []string) error {
+func parseMarkerMultiLine(str string) ([3]string, error) {
+	gr := uniseg.NewGraphemes(str)
+	parts := []string{}
+	totalWidth := 0
+	for gr.Next() {
+		s := string(gr.Runes())
+		totalWidth += uniseg.StringWidth(s)
+		parts = append(parts, s)
+	}
+
+	result := [3]string{}
+	if totalWidth != 3 && totalWidth != 6 {
+		return result, fmt.Errorf("invalid total marker width: %d (expected: 3 or 6)", totalWidth)
+	}
+
+	expected := totalWidth / 3
+	idx := 0
+	for _, part := range parts {
+		expected -= uniseg.StringWidth(part)
+		result[idx] += part
+		if expected <= 0 {
+			idx++
+			expected = totalWidth / 3
+		}
+		if idx == 3 {
+			break
+		}
+	}
+	return result, nil
+}
+
+func parseOptions(index *int, opts *Options, allArgs []string) error {
 	var err error
 	var historyMax int
 	if opts.History == nil {
@@ -1768,10 +1923,16 @@ func parseOptions(opts *Options, allArgs []string) error {
 		opts.Fish = false
 		opts.Help = false
 		opts.Version = false
+		opts.Man = false
 	}
+	startIndex := *index
 	for i := 0; i < len(allArgs); i++ {
 		arg := allArgs[i]
+		index := i + startIndex
 		switch arg {
+		case "--man":
+			clearExitingOpts()
+			opts.Man = true
 		case "--bash":
 			clearExitingOpts()
 			opts.Bash = true
@@ -1787,6 +1948,29 @@ func parseOptions(opts *Options, allArgs []string) error {
 		case "--version":
 			clearExitingOpts()
 			opts.Version = true
+		case "--no-winpty":
+			opts.NoWinpty = true
+		case "--tmux":
+			given, str := optionalNextString(allArgs, &i)
+			if given {
+				if opts.Tmux, err = parseTmuxOptions(str, index); err != nil {
+					return err
+				}
+			} else {
+				opts.Tmux = defaultTmuxOptions(index)
+			}
+		case "--no-tmux":
+			opts.Tmux = nil
+		case "--force-tty-in":
+			// NOTE: We need this because `system('fzf --tmux < /dev/tty')` doesn't
+			// work on Neovim. Same as '-' option of fzf-tmux.
+			opts.ForceTtyIn = true
+		case "--no-force-tty-in":
+			opts.ForceTtyIn = false
+		case "--proxy-script":
+			if opts.ProxyScript, err = nextString(allArgs, &i, ""); err != nil {
+				return err
+			}
 		case "-x", "--extended":
 			opts.Extended = true
 		case "-e", "--exact":
@@ -1914,6 +2098,15 @@ func parseOptions(opts *Options, allArgs []string) error {
 			opts.Tac = true
 		case "--no-tac":
 			opts.Tac = false
+		case "--tail":
+			if opts.Tail, err = nextInt(allArgs, &i, "number of items to keep required"); err != nil {
+				return err
+			}
+			if opts.Tail <= 0 {
+				return errors.New("number of items to keep must be a positive integer")
+			}
+		case "--no-tail":
+			opts.Tail = 0
 		case "-i", "--ignore-case":
 			opts.Case = CaseIgnore
 		case "+i", "--no-ignore-case":
@@ -1962,6 +2155,10 @@ func parseOptions(opts *Options, allArgs []string) error {
 			opts.CursorLine = false
 		case "--no-cycle":
 			opts.Cycle = false
+		case "--multi-line":
+			opts.MultiLine = true
+		case "--no-multi-line":
+			opts.MultiLine = false
 		case "--keep-right":
 			opts.KeepRight = true
 		case "--no-keep-right":
@@ -2049,17 +2246,27 @@ func parseOptions(opts *Options, allArgs []string) error {
 				return err
 			}
 		case "--pointer":
-			str, err := nextString(allArgs, &i, "pointer sign string required")
+			str, err := nextString(allArgs, &i, "pointer sign required")
 			if err != nil {
 				return err
 			}
-			opts.Pointer = firstLine(str)
+			str = firstLine(str)
+			opts.Pointer = &str
 		case "--marker":
-			str, err := nextString(allArgs, &i, "selected sign string required")
+			str, err := nextString(allArgs, &i, "marker sign required")
 			if err != nil {
 				return err
 			}
-			opts.Marker = firstLine(str)
+			str = firstLine(str)
+			opts.Marker = &str
+		case "--marker-multi-line":
+			str, err := nextString(allArgs, &i, "marker sign for multi-line entries required")
+			if err != nil {
+				return err
+			}
+			if opts.MarkerMulti, err = parseMarkerMultiLine(firstLine(str)); err != nil {
+				return err
+			}
 		case "--sync":
 			opts.Sync = true
 		case "--no-sync", "--async":
@@ -2123,7 +2330,7 @@ func parseOptions(opts *Options, allArgs []string) error {
 			if err != nil {
 				return err
 			}
-			if opts.Height, err = parseHeight(str); err != nil {
+			if opts.Height, err = parseHeight(str, index); err != nil {
 				return err
 			}
 		case "--min-height":
@@ -2264,6 +2471,10 @@ func parseOptions(opts *Options, allArgs []string) error {
 				if opts.FuzzyAlgo, err = parseAlgo(value); err != nil {
 					return err
 				}
+			} else if match, value := optString(arg, "--tmux="); match {
+				if opts.Tmux, err = parseTmuxOptions(value, index); err != nil {
+					return err
+				}
 			} else if match, value := optString(arg, "--scheme="); match {
 				opts.Scheme = strings.ToLower(value)
 			} else if match, value := optString(arg, "-q", "--query="); match {
@@ -2291,9 +2502,15 @@ func parseOptions(opts *Options, allArgs []string) error {
 			} else if match, value := optString(arg, "--prompt="); match {
 				opts.Prompt = value
 			} else if match, value := optString(arg, "--pointer="); match {
-				opts.Pointer = firstLine(value)
+				str := firstLine(value)
+				opts.Pointer = &str
 			} else if match, value := optString(arg, "--marker="); match {
-				opts.Marker = firstLine(value)
+				str := firstLine(value)
+				opts.Marker = &str
+			} else if match, value := optString(arg, "--marker-multi-line="); match {
+				if opts.MarkerMulti, err = parseMarkerMultiLine(firstLine(value)); err != nil {
+					return err
+				}
 			} else if match, value := optString(arg, "-n", "--nth="); match {
 				if opts.Nth, err = splitNth(value); err != nil {
 					return err
@@ -2309,7 +2526,7 @@ func parseOptions(opts *Options, allArgs []string) error {
 					return err
 				}
 			} else if match, value := optString(arg, "--height="); match {
-				if opts.Height, err = parseHeight(value); err != nil {
+				if opts.Height, err = parseHeight(value, index); err != nil {
 					return err
 				}
 			} else if match, value := optString(arg, "--min-height="); match {
@@ -2425,11 +2642,19 @@ func parseOptions(opts *Options, allArgs []string) error {
 			} else if match, value := optString(arg, "--jump-labels="); match {
 				opts.JumpLabels = value
 				validateJumpLabels = true
+			} else if match, value := optString(arg, "--tail="); match {
+				if opts.Tail, err = atoi(value); err != nil {
+					return err
+				}
+				if opts.Tail <= 0 {
+					return errors.New("number of items to keep must be a positive integer")
+				}
 			} else {
 				return errors.New("unknown option: " + arg)
 			}
 		}
 	}
+	*index += len(allArgs)
 
 	if opts.HeaderLines < 0 {
 		return errors.New("header lines must be a non-negative integer")
@@ -2471,23 +2696,17 @@ func validateSign(sign string, signOptName string) error {
 	return nil
 }
 
-// This function can have side-effects and alter some global states.
-// So we run it on fzf.Run and not on ParseOptions.
-func postProcessOptions(opts *Options) error {
-	if opts.Ambidouble {
-		uniseg.EastAsianAmbiguousWidth = 2
+func validateOptions(opts *Options) error {
+	if opts.Pointer != nil {
+		if err := validateSign(*opts.Pointer, "pointer"); err != nil {
+			return err
+		}
 	}
 
-	if err := validateSign(opts.Pointer, "pointer"); err != nil {
-		return err
-	}
-
-	if err := validateSign(opts.Marker, "marker"); err != nil {
-		return err
-	}
-
-	if !tui.IsLightRendererSupported() && opts.Height.size > 0 {
-		return errors.New("--height option is currently not supported on this platform")
+	if opts.Marker != nil {
+		if err := validateSign(*opts.Marker, "marker"); err != nil {
+			return err
+		}
 	}
 
 	if opts.Scrollbar != nil {
@@ -2502,6 +2721,72 @@ func postProcessOptions(opts *Options) error {
 		}
 	}
 
+	if opts.Height.auto {
+		for _, s := range []sizeSpec{opts.Margin[0], opts.Margin[2]} {
+			if s.percent {
+				return errors.New("adaptive height is not compatible with top/bottom percent margin")
+			}
+		}
+		for _, s := range []sizeSpec{opts.Padding[0], opts.Padding[2]} {
+			if s.percent {
+				return errors.New("adaptive height is not compatible with top/bottom percent padding")
+			}
+		}
+	}
+
+	return nil
+}
+
+// This function can have side-effects and alter some global states.
+// So we run it on fzf.Run and not on ParseOptions.
+func postProcessOptions(opts *Options) error {
+	if opts.Ambidouble {
+		uniseg.EastAsianAmbiguousWidth = 2
+	}
+
+	if opts.BorderShape == tui.BorderUndefined {
+		opts.BorderShape = tui.BorderNone
+	}
+
+	if opts.Pointer == nil {
+		defaultPointer := "▌"
+		if !opts.Unicode {
+			defaultPointer = ">"
+		}
+		opts.Pointer = &defaultPointer
+	}
+
+	markerLen := 1
+	if opts.Marker == nil {
+		// "▎" looks better, but not all terminals render it correctly
+		defaultMarker := "┃"
+		if !opts.Unicode {
+			defaultMarker = ">"
+		}
+		opts.Marker = &defaultMarker
+	} else {
+		markerLen = uniseg.StringWidth(*opts.Marker)
+	}
+
+	markerMultiLen := 1
+	if len(opts.MarkerMulti[0]) == 0 {
+		if opts.Unicode {
+			opts.MarkerMulti = [3]string{"╻", "┃", "╹"}
+		} else {
+			opts.MarkerMulti = [3]string{".", "|", "'"}
+		}
+	} else {
+		markerMultiLen = uniseg.StringWidth(opts.MarkerMulti[0])
+	}
+	if markerMultiLen > markerLen {
+		padded := *opts.Marker + " "
+		opts.Marker = &padded
+	} else if markerMultiLen < markerLen {
+		for idx := range opts.MarkerMulti {
+			opts.MarkerMulti[idx] += " "
+		}
+	}
+
 	// Default actions for CTRL-N / CTRL-P when --history is set
 	if opts.History != nil {
 		if _, prs := opts.Keymap[tui.CtrlP.AsEvent()]; !prs {
@@ -2548,19 +2833,6 @@ func postProcessOptions(opts *Options) error {
 		opts.Keymap[tui.DoubleClick.AsEvent()] = opts.Keymap[tui.CtrlM.AsEvent()]
 	}
 
-	if opts.Height.auto {
-		for _, s := range []sizeSpec{opts.Margin[0], opts.Margin[2]} {
-			if s.percent {
-				return errors.New("adaptive height is not compatible with top/bottom percent margin")
-			}
-		}
-		for _, s := range []sizeSpec{opts.Padding[0], opts.Padding[2]} {
-			if s.percent {
-				return errors.New("adaptive height is not compatible with top/bottom percent padding")
-			}
-		}
-	}
-
 	// If we're not using extended search mode, --nth option becomes irrelevant
 	// if it contains the whole range
 	if !opts.Extended || len(opts.Nth) == 1 {
@@ -2589,12 +2861,22 @@ func postProcessOptions(opts *Options) error {
 		theme.Spinner = boldify(theme.Spinner)
 	}
 
+	// If --height option is not supported on the platform, just ignore it
+	if !tui.IsLightRendererSupported() && opts.Height.size > 0 {
+		opts.Height = heightSpec{}
+	}
+
+	if err := opts.initProfiling(); err != nil {
+		return errors.New("failed to start pprof profiles: " + err.Error())
+	}
+
 	return processScheme(opts)
 }
 
 // ParseOptions parses command-line options
 func ParseOptions(useDefaults bool, args []string) (*Options, error) {
 	opts := defaultOptions()
+	index := 0
 
 	if useDefaults {
 		// 1. Options from $FZF_DEFAULT_OPTS_FILE
@@ -2609,7 +2891,7 @@ func ParseOptions(useDefaults bool, args []string) (*Options, error) {
 				return nil, errors.New(path + ": " + parseErr.Error())
 			}
 			if len(words) > 0 {
-				if err := parseOptions(opts, words); err != nil {
+				if err := parseOptions(&index, opts, words); err != nil {
 					return nil, errors.New(path + ": " + err.Error())
 				}
 			}
@@ -2621,19 +2903,20 @@ func ParseOptions(useDefaults bool, args []string) (*Options, error) {
 			return nil, errors.New("$FZF_DEFAULT_OPTS: " + parseErr.Error())
 		}
 		if len(words) > 0 {
-			if err := parseOptions(opts, words); err != nil {
+			if err := parseOptions(&index, opts, words); err != nil {
 				return nil, errors.New("$FZF_DEFAULT_OPTS: " + err.Error())
 			}
 		}
 	}
 
 	// 3. Options from command-line arguments
-	if err := parseOptions(opts, args); err != nil {
+	if err := parseOptions(&index, opts, args); err != nil {
 		return nil, err
 	}
 
-	if err := opts.initProfiling(); err != nil {
-		return nil, errors.New("failed to start pprof profiles: " + err.Error())
+	// 4. Final validation of merged options
+	if err := validateOptions(opts); err != nil {
+		return nil, err
 	}
 
 	return opts, nil
